﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DimaCreate.XDima;

namespace DimaCreate
{
    class Program
    {
        static void Main(string[] args)
        {
            // Step 1: Initialize PTV xDima client
		    XDimaWSService xDimaClient = new XDimaWSService();

		    // Step 2: Create an unique dima ID
		    int dimaID = 1001;

		    // Step 3.1: Define a new dima profile and set a bundle of parameters
		    DimaProfile dimaProfile = new DimaProfile();
		    String description = "PTV code sample";
		    dimaProfile.description = description;

		    // Step 3.2: Set countryCodes
		    short[] countryCodes = new short[1];
		    countryCodes[0] = 52;
		    CountryInfo countryInfo = new CountryInfo();
		    countryInfo.wrappedCountryCodes = countryCodes;
		    dimaProfile.countryInfo = countryInfo;

		    // Step 3.3: Set the correct coordinate format
		    dimaProfile.dimaCoordFormat = DimaCoordFormat.MERCATOR;

		    // Step 3.4: Avoid or prefer some elements
		    short ferryMalus = 100;
		    dimaProfile.ferryMalus = ferryMalus;
            dimaProfile.ferryMalusSpecified = true;
		    short highwayMalus = 0;
		    dimaProfile.highwayMalus = highwayMalus;
            dimaProfile.highwayMalusSpecified = true;
		    short sideRoadMalus = 0;
		    dimaProfile.sideRoadMalus = sideRoadMalus;
            dimaProfile.sideRoadMalusSpecified = true;
		    short urbanAreaMalus = 500;
		    dimaProfile.urbanAreaMalus = urbanAreaMalus;
            dimaProfile.urbanAreaMalusSpecified = true;
		    short residentsOnlyMalus = 2000;
		    dimaProfile.residentsOnlyMalus = residentsOnlyMalus;
            dimaProfile.residentsOnlyMalusSpecified = true;
		    short rampMalus = 10;
		    dimaProfile.rampMalus = rampMalus;
            dimaProfile.rampMalusSpecified = true;
		    short tollRoadMalus = 0;
		    dimaProfile.tollRoadMalus = tollRoadMalus;
            dimaProfile.tollRoadMalusSpecified = true;

		    // Step 3.5: Set the existing vehicle type
		    dimaProfile.vehicleType = VehicleType.VTCAR;
		
		    // Step 3.6: Set the maximum number of locations
		    short maxLocationCount = 5;
		    dimaProfile.maxLocationCount = maxLocationCount;
		
		    // Step 3.7: Select the linking type
		    dimaProfile.linkType = LinkType.LTSEGMENT;
	
		    // Step 3.8: Select the routing type
		    dimaProfile.routingType = RoutingType.RT1TON;
		
		    // Step 3.9: Add the speed profile as 8 pairs for maximum and minimum speed of 8 network classes.
		    String speedTable = "135,70,125,35,85,25,60,25,50,20,40,18,16,9,6,4";
		    dimaProfile.speedTable = speedTable;
		
		    // Step 3.10 Set the value for the optimization factor
		    short optimization = 80;
		    dimaProfile.optimization = optimization;

		    try 
            {
			    Console.WriteLine("Start to create Dima...");
                // Step 4: Call method createDima()
			    xDimaClient.createDima(dimaID, dimaProfile, null);
			
			    // Step 5: Dima is created and lies in the corresponding folder of PTV xDima Server.  
                Console.WriteLine("Dima <ID=1001> is created. Check your dima folder for output.");
		    } 
            catch (Exception e) 
            {
                // TODO: proper exception handling
            }
        }
    }
}
