﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GeocodingBasic.XLocate;

namespace GeocodingBasic
{
    class Program
    {
        static void Main(string[] args)
        {
            // Step 1: Instantiate XLocate client
            XLocateWSService client = new XLocateWSService();

            // Step 2.1: Set CallerContext for adding log information into your basic response logs.
            CallerContext context = new CallerContext();
            context.log1 = "Basic Geocoding";

            // Step 2.2 Set CallerContext properties to define profile and CoordFormat
            CallerContextProperty prop1 = new CallerContextProperty();
            prop1.key = "Profile";
            prop1.value = "default";
            CallerContextProperty prop2 = new CallerContextProperty();
            prop2.key = "CoordFormat";
            prop2.value = "PTV_MERCATOR";
            CallerContextProperty[] props = new CallerContextProperty[] {prop1, prop2};
            context.wrappedProperties = props;

            // Step 3: Create address object
            Address addr = new Address
            {
                country = "L",
                postCode = "1856",
                city = "Luxemburg",
                street = "Rue Evrard Ketten",
                houseNumber = "40"
            };

            try 
			{
			    // Step 4: Get address response, pass the CallerContext as a parameter
                AddressResponse response = client.findAddress(addr, null, null, null, context);

                // Step 5: Output the results to the console
                foreach (ResultAddress ra in response.wrappedResultList)
                {
                    Console.WriteLine(
                        String.Format("Found {0}~{1}~{2}~{3}~{4} at location: x={5} y={6}, total score is: {7}",
                            ra.country, ra.postCode, ra.city, ra.street, ra.houseNumber, ra.coordinates.point.x, ra.coordinates.point.y, ra.totalScore)
                    );
                }
                Console.WriteLine("Press any key to exit...");
                Console.ReadLine();
			} 
            catch (Exception e)
			{
				// TODO: proper exception handling
			}
        }
    }
}
