﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using GeocodingWithSearchOptions.XLocate;

namespace GeocodingWithSearchOptions
{
    class Program
    {
        static void Main(string[] args)
        {
            // Step 1: Instantiate XLocate client
            XLocateWSService client = new XLocateWSService();

            // Step 2.1: Set CallerContext for adding log information into your basic response logs.
            CallerContext context = new CallerContext();
            context.log1 = "Geocoding with search options";

            // Step 2.2 Set CallerContext properties to define profile and CoordFormat
            CallerContextProperty prop1 = new CallerContextProperty();
            prop1.key = "Profile";
            prop1.value = "default";
            CallerContextProperty prop2 = new CallerContextProperty();
            prop2.key = "CoordFormat";
            prop2.value = "PTV_MERCATOR";
            CallerContextProperty[] props = new CallerContextProperty[] { prop1, prop2 };
            context.wrappedProperties = props;

            // Step 3: Create address object
            Address addr = new Address();
            addr.country = "LUX";
            addr.postCode = "1856";
            addr.city = "Luxemburg";
            addr.street = "Rue Evrard Ketten";
            addr.houseNumber = "40";

            // Step 4: Create SearchOptions and use corresponding parameters
            //         Specify the format in which the country code is passed, 2 = ISO-3
            //         Please note that the country code given in the address above must follow this format
            SearchOption soCountryCodeType = new SearchOption();
            soCountryCodeType.param = SearchParameter.COUNTRY_CODETYPE;
            soCountryCodeType.value = "2";

            // Specify the language of the displayed results
            SearchOption soResultLanguage = new SearchOption();
            soResultLanguage.param = SearchParameter.RESULT_LANGUAGE;
            soResultLanguage.value = "FRE";

            SearchOption[] options = new SearchOption[] { soCountryCodeType, soResultLanguage };

            try
            {
                // Step 5: Get address response, pass the SearchOptions as a parameter
                AddressResponse response =
                    client.findAddress(addr, options, null, null, context);

                // Step 6: Output the results to the console
                foreach (ResultAddress ra in response.wrappedResultList)
                {
                    Console.WriteLine(
                        String.Format("Found {0}~{1}~{2}~{3}~{4} at location: x={5} y={6}, total score is: {7}",
                            ra.country, ra.postCode, ra.city, ra.street, ra.houseNumber, ra.coordinates.point.x, ra.coordinates.point.y, ra.totalScore)
                    );
                }
                Console.WriteLine("Press any key to exit...");
                Console.ReadLine();
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
            }
        }
    }
}
