﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingBasic.XMap;

namespace MappingBasic
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        public void runSample()
        {
            // step 1:
		    // instantiate remote interface for xMap
		    XMapWSService client = new XMapWSService();

		    // step 2:
		    // create object MapSection
		    MapSection mapSection = new MapSection();
		    MappingBasic.XMap.Point point = new MappingBasic.XMap.Point();
            PlainPoint plainpoint = new PlainPoint();
            plainpoint.x = 685407.751736;
            plainpoint.y = 6372537.965244;
            point.point = plainpoint;
		    mapSection.center = point;
		    mapSection.scale = 1000;
		
		    // step 3:
		    // create object CallerContext
		    CallerContext cxt = new CallerContext();
		    cxt.log1 = "Basic Mapping";
		    // set CallerContext properties to define profile and CoordFormat
		    CallerContextProperty props1 = new CallerContextProperty();
		    props1.key = "Profile";
		    props1.value = "default";
		    CallerContextProperty props2 = new CallerContextProperty();
		    props2.key = "CoordFormat";
		    props2.value = "PTV_MERCATOR";
		    CallerContextProperty[] properties = new CallerContextProperty[]{props1, props2};
		    cxt.wrappedProperties = properties;

		    // step 4:
		    // create object ImageInfo
		    ImageInfo imageInfo = new ImageInfo();
		    imageInfo.height = 315;
		    imageInfo.width = 515;
		    imageInfo.imageParameter = "";

		    // step 5:
		    // create object MapParams
		    MapParams mapParams = new MapParams();
		    mapParams.showScale = true;
		    mapParams.useMiles = false;

		    // step 6:
		    // set attribute includeImageInResponse
            // please note that your imageMapServer.remoteURL in the conf/xmap.properties
            // is set correctly, if this paramater is false
		    bool includeImageInResponse = false;
		
			try
            {
                // step 7:
			    // render map
                Map map = client.renderMap(mapSection, mapParams, imageInfo, null, includeImageInResponse, cxt);

			    // step 8:
			    // retrieve further map values
			    BoundingBox bbox = map.visibleSection.boundingBox; 
			    double leftTopX = bbox.leftTop.point.x; 
			    double leftTopY =	bbox.leftTop.point.y; 
			    double rightBottomX =	bbox.rightBottom.point.x; 
			    double rightBottomY =	bbox.rightBottom.point.y;
			    int scale = map.visibleSection.scale;

                // display text info
                String displayText = String.Format("Top-left corner: ({0}, {1}) \nBottom-right corner: ({2}, {3}) \nScale: {4}", leftTopX, leftTopY, rightBottomX, rightBottomY, scale);
                text1.Text = displayText;

			    // step 9:
			    // get the image url
			    String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
