﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithFootprints.XMap;

namespace MappingWithFootprints
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        public void runSample()
        {
            // Step 1: Initialize xmap client
            XMapWSService xMapClient = new XMapWSService();

            // Step 2: Create object MapSection
            MapSection mapSection = new MapSection();
            XMap.Point point = new XMap.Point();
            point.point = new PlainPoint { x = 681490.0, y = 6372500.0 };
            mapSection.center = point;
            mapSection.scale = 10;

            // Step 3: Create object CallerContext
            CallerContext cxt = new CallerContext();
            cxt.log1 = "Basic Mapping";
            // Step 3.1: Set CallerContext properties to define profile "footprints"
            CallerContextProperty props1 = new CallerContextProperty
            {
                key = "Profile",
                value = "footprints"
            };
            // Step 3.2: Set CallerContext properties to define coordinate format "PTV Mercator"
            CallerContextProperty props2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR"
            };
            // Step 3.3: Put properties into class CallerContextProperty[]
            CallerContextProperty[] properties = new CallerContextProperty[] { props1, props2 };
            cxt.wrappedProperties = properties;
            // Step 3.4: set CallerContext

            // Step 4: Create object ImageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 5: Create object MapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 6: Set attribute includeImageInResponse
            bool includeImageInResponse = false;

            try
            {
                // Step 7: Render map
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, null, includeImageInResponse, cxt);

                // Step 8: Get the image from object map
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
