﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithLanguage.XMap;

namespace MappingWithLanguage
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        XMapWSService xMapClient;

        public MainWindow()
        {
            // Step 1: Initialize xmap client
            xMapClient = new XMapWSService();
            InitializeComponent();
            // set initial language selection and load the map
            comboLanguage.SelectedIndex = 0;
            updateMap();
        }

        private void updateMap()
        {
            // Step 2: Create object mapSection
            MapSection mapSection = new MapSection();
            XMap.Point point = new XMap.Point { point = new PlainPoint { x = 681600, y = 6372500 } };
            mapSection.center = point;
            mapSection.scale = 20000;

            // Step 3: Set the parameters for imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 4: Define class mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 5: Use class CallerContext to define the specific profile
            CallerContext ctx = new CallerContext();
            CallerContextProperty ctx_property = new CallerContextProperty();
            ctx_property.key = "Profile";
            // Note: Did you add the new profiles before and restarted the PTV xMap Server?
            // Step 6: Select the profile via application
            ctx_property.value = (comboLanguage.SelectedValue as ComboBoxItem).Content.ToString();
            CallerContextProperty[] properties = { ctx_property };
            ctx.wrappedProperties = properties;

            // Step 7: Set boolean value to false for attribute includeImageInResponse
            bool includeImageInResponse = false;

            try
            {
                // Step 8: Return map object with the specific information
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, null, includeImageInResponse, ctx);

                // Step 9: Retrieve the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO proper exception handling
            } 
        }

        private void comboLanguage_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Set Url for xRoute
            ComboBox comboBox = sender as ComboBox;
            updateMap();
        }
    }
}
