﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace MappingWithRoute
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample() {
		    // Step 1: initialize clients for xMap and xRoute. Use the utility classes to define the clients.
		    XMapWSService xMapClient = new XMapWSService();

		    XRouteWSService xRouteClient = new XRouteWSService();

            try
            {
                // Step 2: calculate a route
                Route route = calculateRoute(xRouteClient);
                // Step 3: map the route
                displayMapOfRoute(xMapClient, route);
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
	    }
        
        /**
	    * Calculates and it returns a route that is suitable for mapping.
	    * 
	    * It uses xRoute to calculate the route.
	    */
	    private Route calculateRoute(XRouteWSService xRouteClient) {

		    // Step 2.1: Specify the ResultListOptions
		    ResultListOptions resultListOptions = new ResultListOptions 
                            { polygon = true, totalRectangle = true, detailLevel = DetailLevel.STANDARD};

		    // Step 2.2: Set WKB (well-known binary) as geometry format
		    CallerContextProperty property = new CallerContextProperty { key = "ResponseGeometry", value = "WKB" };
		    CallerContext callerContext = new CallerContext();
		    callerContext.wrappedProperties = new CallerContextProperty[] { property };

		    // Step 2.3: Get the way points from the getWaypoints() function
		    WaypointDesc[] waypointDescs = getWaypoints();

		    // Step 2.4: Return object route as a polygon using calculateRoute() function
		    Route route = xRouteClient.calculateRoute(waypointDescs, null, null, resultListOptions, callerContext);

		    return route;
	    }

	    /**
	    * Uses xMap to create a map of a route and displays it in a JFrame
	    */
	    private void displayMapOfRoute(XMapWSService xMapClient, Route route) {

		    // Step 3.1: Create imageInfo and set the frame size and image format.
		    ImageInfo imageInfo = new ImageInfo();
		    imageInfo.format = ImageFileFormat.GIF;
		    imageInfo.height = 600;
		    imageInfo.width = 800;
		    imageInfo.imageParameter = "";

		    // Step 3.2: Build mapParams
		    MapParams mapParams = new MapParams();
		    mapParams.showScale = true;
		    mapParams.useMiles = true;
		    mapParams.language = "x-ptv-DFT"; //optional
		
		    // Step 3.3: Set argument includeImageInResponse to false (default).
		    bool includeImageInResponse = false;

		    // Step 3.4: Create a line from the calculated route;
            LineString[] lineStrings = new LineString[] { route.polygon };
		    Lines[] lines = new Lines[1];
		    LineOptions options = new LineOptions();
		    LinePartOptions partoptions = new LinePartOptions();
            partoptions.color = new xserver.Color { red = 0, green = 0, blue = 255 };
		    partoptions.visible = true;
		    partoptions.width = -10;
		    options.mainLine = partoptions;

		    lines[0] = new Lines { wrappedLines = lineStrings, options = options };

		    // Step 3.5: Define customLayer that contains the object lines and set layers.
		    CustomLayer customLayer = new CustomLayer();
		    customLayer.visible = true;
		    customLayer.drawPriority = 100; // this is PRIORROUTE
		    customLayer.wrappedLines = lines;
		    customLayer.objectInfos = ObjectInfoType.NONE;
		    customLayer.centerObjects = true;
		    Layer[] layers = new Layer[] { customLayer };
		
		    // Step 3.6: Set boundingBox fand use corners from the calculated route
		    BoundingBox boundingBox = new BoundingBox();
		    boundingBox.leftTop = route.totalRectangle.leftBottom;
		    boundingBox.rightBottom = route.totalRectangle.rightTop;

            // Step 4: call renderMapBoundingBox and return a valid object map.
            Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, null);

            // Step 5: retrieve and display the image
            String imageUrl = "http://" + map.image.url;
            // Create BitmapImage
            BitmapImage bitmapImage = new BitmapImage();

            // Display BitmapImage
            bitmapImage.BeginInit();
            bitmapImage.UriSource = new Uri(imageUrl);
            bitmapImage.EndInit();

            img1.Source = bitmapImage;
	    }

	    /**
	    * Returns a list of pre-configured waypoints.
	    */
	    private WaypointDesc[] getWaypoints() {
		    // Step 2.3.1: Set points = list of pairs of (x,y) coordinates for start and destination
		    double[][] points = new double[][] { new double[] { 678730.0, 6372820.0 },
				    new double[] { 683080.0, 6372430.0 } };

		    // Step 2.3.2: Define waypointDescs, set points and link type as next segment
		    WaypointDesc[] waypointDescs = new WaypointDesc[points.Length];
		    for (int i = 0; i < points.Length; i++) {
			    xserver.Point pt = new xserver.Point();
			    pt.point = new PlainPoint { x = points[i][0], y = points[i][1]};

			    waypointDescs[i] = new WaypointDesc();
			    waypointDescs[i].wrappedCoords = new xserver.Point[] { pt };
			    waypointDescs[i].linkType = LinkType.NEXT_SEGMENT;
		    }
		    return waypointDescs;
	    }
    }
}
