﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MappingWithTextToMap.XMap;

namespace MappingWithTextToMap
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Initialize xmap client
            XMapWSService xMapClient = new XMapWSService();
            // Step 2: Define mapSection. Set center and scale.
            MapSection mapSection = new MapSection();
            XMap.Point point = new XMap.Point();
            point.point = new PlainPoint { x = 685407.751736, y = 6372537.965244 };
            mapSection.center = point;
            mapSection.scale = 1500;

            // Step 3: Create object imageInfo and set the relevant parameters
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 4: Add new class mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;

            // Step 5: Set text blocks and place object via defined location
            XMap.Point point1 = new XMap.Point();
            point1.wkt = "POINT(681254.0 6378399.0)";
            Text text1 = new Text();
            text1.position = point1;
            text1.text = "Start point";

            XMap.Point point2 = new XMap.Point();
            point2.wkt = "POINT(688875.0 6369972.0)";
            Text text2 = new Text();
            text2.text = "Destination point";
            text2.position = point2;

            // Step 6: Set text options
            Font font = new Font
            {
                name = "Arial",
                size = 24,
                bold = true,
            };

            XMap.TextOptions textOptions = new XMap.TextOptions
            {
                font = font,
                textColor = new XMap.Color { red = 0, green = 0, blue = 0 },
                fillBg = true,
                bgColor = new XMap.Color { red = 255, green = 255, blue = 255 },
                showFrame = true,
                frameColor = new XMap.Color { red = 0, green = 0, blue = 255 },
                alignment = XMap.TextAlignment.CENTER,
                pixelX = 0,
                pixelY = 0,
                relX = 0,
                relY = 0
            };

            // Step 7: If you want to style each text object differently
            // you should create an object "Texts" for each text.
            // This allows you to use different text options.
            Texts[] texts = new Texts[] {
                new Texts {
                    options = textOptions,
                    wrappedTexts = new Text[] { text1, text2 }
                }
            };

            // Step 8: Define customLayer and set the texts visible
            CustomLayer customLayer = new CustomLayer
            {
                drawPriority = 25000,
                name = "Text layer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedTexts = texts
            };
            Layer[] layers = new Layer[] { customLayer };

            // Step 9: Set value to false
            bool includeImageInResponse = false;

            try
            {
                // Step 10: Return object map
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 11: Retrieve the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
