﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingAvoidStreets
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Initialize clients
            XMapWSService xMapClient = new XMapWSService();
            XRouteWSService xRouteClient = new XRouteWSService();

            //Step 2: Define start and destination
            xserver.Point startPoint = new xserver.Point();
            startPoint.point = new PlainPoint { x = 842716, y = 4900805 } ;

            WaypointDesc wpdStart = new WaypointDesc();
            wpdStart.wrappedCoords = new xserver.Point[] { startPoint };
            wpdStart.linkType = LinkType.NEXT_SEGMENT;

            // destination
            xserver.Point destPoint = new xserver.Point();
            destPoint.point = new PlainPoint { x = 843793, y = 4900919 };

            WaypointDesc wpdDest = new WaypointDesc();
            wpdDest.wrappedCoords = new xserver.Point[] { destPoint };
            wpdDest.linkType = LinkType.NEXT_SEGMENT;

            WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };
            
            // Step 3: Define which results to get
            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true,
                totalRectangle = true,
                detailLevel = DetailLevel.STANDARD
            };

            // Step 4: Defining ExceptionPaths
            xserver.ExceptionPath[] exceptionPaths = new xserver.ExceptionPath[] 
            { new xserver.ExceptionPath {
                    street = "Gerwigstraße",
                    relMalus = 2500,
                    absTimeMalus = -1
                }
            };

            // you can list multiple streets like this
            // street = "Gerwigstraße/Tullastrasse/Esseweinstrasse";

            // OR strongly prefer street
            /*
             * street = "Striederstraße";
             * relMalus = -100;
             * absTimeMalus = -1;
             */

            // Step 5: set coordinate format in caller context to GeoDecimal
            CallerContextProperty coordFormat = new CallerContextProperty
            {
                key = "CoordFormat", 
                value = "PTV_GEODECIMAL"
            };
            CallerContext callerContext = new CallerContext();
            callerContext.wrappedProperties = new CallerContextProperty[] { coordFormat };

            try
            {
                // Step 6: Calculate route with exceptions ( streets with malus or bonus )
                Route route_withException = xRouteClient.calculateRoute(
                        waypointDescs, null, exceptionPaths, resultListOptions, callerContext);

                // Step 7: calculate default route (without exceptions) for comparison
                Route defaultRoute = xRouteClient.calculateRoute(waypointDescs,
                        null, null, resultListOptions, callerContext);

                // Step 8: Create map with routes
                // Step 8.1: Use bounding box of route
                BoundingBox boundingBox = new BoundingBox();
                boundingBox.leftTop = route_withException.totalRectangle.leftBottom;
                boundingBox.rightBottom = route_withException.totalRectangle.rightTop;

                // Step 8.2: Get "polygon" from route
                LineString[] lineStrings1 = new LineString[] { route_withException.polygon };

                // Step 8.3: Set line options for alternative route( color, stroke size )
                LineOptions options1 = new LineOptions();
                LinePartOptions linePartOpt1 = new LinePartOptions
                {
                    color = new xserver.Color { red = 0, green = 255, blue = 0 },
                    visible = true,
                    width = -6
                };
                options1.mainLine = linePartOpt1;

                // Step 8.4: Same with default route, just different color
                LineString[] lineStrings2 = new LineString[] { defaultRoute.polygon };
                LineOptions options2 = new LineOptions();
                LinePartOptions linePartOpt2 = new LinePartOptions
                {
                    color = new xserver.Color { red = 0, green = 0, blue = 255 },
                    visible = true,
                    width = -6
                };
                options2.mainLine = linePartOpt2;

                Lines[] linesArray = new Lines[] {
					new Lines { wrappedLines = lineStrings1, options = options1},
					new Lines { wrappedLines = lineStrings2, options = options2} 
                };

                // Step 8.5: put lines into a CustomLayer
                CustomLayer customLayer = new CustomLayer
                {
                    name = "routeLayer",
                    wrappedLines = linesArray,
                    drawPriority = 100, // this is PRIORROUTE
                    centerObjects = true,
                    objectInfos = ObjectInfoType.FULLGEOMETRYCLIPPED,
                    visible = true
                };

                // Step 8.6: Create Image info and set map parameters
                ImageInfo imageInfo = new ImageInfo
                {
                    format = ImageFileFormat.GIF,
                    height = 600,
                    width = 800,
                    imageParameter = ""
                };

                MapParams mapParams = new MapParams { showScale = false, useMiles = false, language = "x-ptv-DFT" };

                // Step 9: Generate map
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams,
                        imageInfo, new Layer[] { customLayer }, false, callerContext);

                // Step 8: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Step 10: Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;

            }
            catch (Exception e)
            {
                // TODO proper exception handling
            }
        }
    }
}
