﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithAlternative
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            List<Route> routes = getRoutes();
            if (routes != null)
                displayMapOfRoute(routes);
        }

        private List<Route> getRoutes()
        {

            // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();

            // Step 2: Set start and destination
            xserver.Point startPoint = new xserver.Point();
            startPoint.point = new PlainPoint { x = 681593.765751686, y = 6371793.192390807 };

            WaypointDesc wpdStart = new WaypointDesc();
            wpdStart.wrappedCoords = new xserver.Point[] { startPoint };
            wpdStart.linkType = LinkType.NEXT_SEGMENT;

            xserver.Point destPoint = new xserver.Point();
            destPoint.point = new PlainPoint { x = 678909.5202224851, y = 6395906.712571196 };

            WaypointDesc wpdDest = new WaypointDesc();
            wpdDest.wrappedCoords = new xserver.Point[] { destPoint };
            wpdDest.linkType = LinkType.NEXT_SEGMENT;

            WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdDest };

            // Step 3: Specify the ResultListOptions
            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true,
                totalRectangle = true,
                binaryPathDesc = true,
                detailLevel = DetailLevel.STANDARD,
            };

            try
            {
                // Step 4: Calculate a first route
                Route firstRoute = xRouteClient.calculateRoute(waypointDescs, null,
                        null, resultListOptions, null);

                // Step 5: take the calculated route and put it into an exception path
                //         and set a malus to the supplied route
                ExceptionPath exceptionPath = new ExceptionPath();
                exceptionPath.binaryPathDesc = firstRoute.binaryPathDesc;
                exceptionPath.relMalus = 500;

                ExceptionPath[] arrayOfExceptionPaths = new ExceptionPath[] { exceptionPath };

                // Step 6: Calculate an alternative route
                Route alternativeRoute = xRouteClient.calculateRoute(waypointDescs, null,
                        arrayOfExceptionPaths, resultListOptions, null);

                // Step 7: Create result list
                List<Route> results = new List<Route> { firstRoute, alternativeRoute };
                return results;
            }
            catch (Exception e)
            {
                // proper exception handling
                return null;
            }

        }

        private void displayMapOfRoute(List<Route> routes)
        {
            // Step 8: Instantiate xRoute client
            XMapWSService xMapClient = new XMapWSService();
            // Step 9.1: Create imageInfo and set the frame size and image format.
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 9.2: Build mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = true;
            mapParams.language = "x-ptv-DFT"; //optional

            // Step 9.3: Set argument includeImageInResponse to false (default).
            bool includeImageInResponse = false;

            Lines[] lines = new Lines[2];
            // Step 9.4: Create lines from the calculated routes
            int i = 0;
            foreach (Route route in routes)
            {
                LineString[] lineStrings = new LineString[] { route.polygon };
                LineOptions options = new LineOptions();
                LinePartOptions partoptions = new LinePartOptions();
                // set colour for first and second route
                if (i == 0)
                    partoptions.color = new xserver.Color { red = 0, green = 0, blue = 255 };
                else
                    partoptions.color = new xserver.Color { red = 255, green = 0, blue = 0 };
                partoptions.visible = true;
                partoptions.width = -10;
                options.mainLine = partoptions;
                lines[i] = new Lines { wrappedLines = lineStrings, options = options };
                i++;
            }


            // Step 9.5: Define customLayer that contains the object lines and set layers.
            CustomLayer customLayer = new CustomLayer();
            customLayer.visible = true;
            customLayer.drawPriority = 100; // this is PRIORROUTE
            customLayer.wrappedLines = lines;
            customLayer.objectInfos = ObjectInfoType.NONE;
            customLayer.centerObjects = true;
            Layer[] layers = new Layer[] { customLayer };

            // Step 9.6: Set boundingBox and use corners from the calculated route
            double top = double.MinValue;
            double bottom = double.MaxValue;
            double left = double.MaxValue;
            double right = double.MinValue;
            foreach (Route route in routes)
            {
                if (route.totalRectangle.leftBottom.point.x < bottom)
                    bottom = route.totalRectangle.leftBottom.point.x;
                if (route.totalRectangle.leftBottom.point.x > top)
                    top = route.totalRectangle.leftBottom.point.x;
                if (route.totalRectangle.leftBottom.point.y < left)
                    left = route.totalRectangle.leftBottom.point.y;
                if (route.totalRectangle.leftBottom.point.y > right)
                    right = route.totalRectangle.leftBottom.point.y;
            }

            BoundingBox boundingBox = new BoundingBox
            {
                leftTop = new xserver.Point { point = new xserver.PlainPoint { x = top, y = left } },
                rightBottom = new xserver.Point { point = new xserver.PlainPoint { x = bottom, y = right } }
            };

            // Step 10: call renderMapBoundingBox and return a valid object map.
            Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, null);

            // Step 11: retrieve and display the image
            String imageUrl = "http://" + map.image.url;
            // Create BitmapImage
            BitmapImage bitmapImage = new BitmapImage();

            // Display BitmapImage
            bitmapImage.BeginInit();
            bitmapImage.UriSource = new Uri(imageUrl);
            bitmapImage.EndInit();

            img1.Source = bitmapImage;
        }
    }
}
