﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using RoutingWithCountryInfo.XRoute;

namespace RoutingWithCountryInfo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static String[] networkClassNames = new String[] {
			 "MOTORWAY", "HIGHWAY", "TRUNK_ROAD",
			 "COUNTRY_ROAD", "CITY_ROAD", "RESIDENTIAL_ROAD",
			 "SPECIAL_ROAD", "CYCLE_AND_WALKWAY" };

        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: instantiate xRoute client.
            XRouteWSService xRouteClient = new XRouteWSService();

		    // Step 2: Set start and destination
            // start
		    XRoute.Point startPoint = new  XRoute.Point();
		    startPoint.point = new PlainPoint { x = 681593.765751686, y = 6371793.192390807 } ;

		    WaypointDesc wpdStart = new WaypointDesc();
		    wpdStart.wrappedCoords = new  XRoute.Point[]{startPoint};
		    wpdStart.linkType = LinkType.NEXT_SEGMENT;

		    // destination
            XRoute.Point destPoint = new  XRoute.Point();
		    destPoint.point = new PlainPoint { x = 678909.5202224851, y = 6395906.712571196 } ;

		    WaypointDesc wpdDest = new WaypointDesc();
		    wpdDest.wrappedCoords = new  XRoute.Point[]{destPoint};
		    wpdDest.linkType = LinkType.NEXT_SEGMENT;

		    WaypointDesc[] waypointDescs = new WaypointDesc[] {wpdStart, wpdDest};

		    // Step 3: Specify the ResultListOptions. They provide the information, e.g. how detailed the itinerary  
		    //         should be. 
		    ResultListOptions resultListOptions = new ResultListOptions();
		    resultListOptions.detailLevel = DetailLevel.STANDARD;

		    try {
			    // Step 4: Calculate the route
                ExtendedRoute route = xRouteClient
				    .calculateExtendedRoute(waypointDescs, null, null, resultListOptions, null, null);

                List<RoutingTableEntry> rows = new List<RoutingTableEntry>();

                // Step 5: Create the table entries
			    foreach (CountryInfo info in route.wrappedCountryInfos ){
                    RouteInfo[] routeInfo = info.wrappedPerNCRouteInfo;
				
				    // get distance for each network class (street type)
				    for (int j=0; j < networkClassNames.Length; j++) {
                        RoutingTableEntry row = new RoutingTableEntry
                        {
                            code = info.iuCode,
                            nc = networkClassNames[j],
                            dist = routeInfo[j].distance / 1000.0,
                            time = routeInfo[j].time / 60.0
                        };
                        rows.Add(row);
				    }
			    }

                gridRoute.ItemsSource = rows;
	
		    }catch(Exception e){
			    // proper exception handling
		    }
        }
    }

    public class RoutingTableEntry
    {
        public int code { get; set; }
        public String nc { get; set; }
        public double dist { get; set; }
        public double time { get; set; }
    }
}
