﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using RoutingWithEmissions.XRoute;

namespace RoutingWithEmissions
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample() {
            // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();
		
		    // Step 2: Use truck profile for toll cost information in the CallerContext  
		    CallerContextProperty[] props = new CallerContextProperty[3];
		    props[0] = new CallerContextProperty { key = "Profile", value = "truckfast" }; 
		    props[1] = new CallerContextProperty { key = "CoordFormat", value = "PTV_MERCATOR" }; 
		    props[2] = new CallerContextProperty { key = "ResponseGeometry", value = "PLAIN"}; 
		
		    // Step 3: create a CallerContext and add the property
		    CallerContext callerContext = new CallerContext();
		    callerContext.wrappedProperties =  props ;

		
		    double[][] points = new double[][]{
		    //Luxembourg coordinates used for sample route
				    new double[] {670182, 6464052},
				    new double[] {682299, 6372572},
				    new double[] {665180, 6354110},
				    new double[] {670182, 6464052},
				    new double[] {665180, 6354110}
				
		    // If Europe City Map Premium is installed you can use the following coordinates:		
			    /*  {  262175.0, 6242590.0}, // Paris
				    {  933441.0, 6268969.0}, // Karlsruhe
				    { 1820467.0, 6134685.0}, //Wien
				    { 1267389.0, 5978788.0}, //Innsbruck
				    { 1262552.0, 5853474.0} //Bolzano*/
		    };
		
		    WaypointDesc[] waypointDescs  = new WaypointDesc[points.Length];
		    for (int i=0; i < points.Length; i++) {
			    XRoute.Point pt = new XRoute.Point();
			    pt.point = new PlainPoint { x = points[i][0], y = points[i][1] };
			
			    waypointDescs[i] = new WaypointDesc();
			    waypointDescs[i].wrappedCoords = new XRoute.Point[]{pt};
			    waypointDescs[i].linkType = LinkType.AUTO_LINKING;
		    }
		
		    // Step 4: Specify the HBEFA version
		    // Enable the parameter for HBEFA 2.1 (use version 2.1; version 3.1 needs additional data)
		    // Enable the path for HBEFA 2.1
		    // Check license key: xroute.emissions=true
		    // Restart xRoute server
		    HBEFAType hbefaType = new HBEFAType();
		    hbefaType.version = HBEFAVersion.HBEFA_2_1;
		
		    // Step 5:Specify the ResultListOptions. They provide the information,   
		    // e.g. how detailed the itinerary should be. 
		    EmissionType emissions = new EmissionType();
		    emissions.emissionLevel = EmissionLevel.STATIONS;
            ResultListOptions resultListOptions = new ResultListOptions 
            {
                hbefaType = hbefaType,
                emissions = emissions
            };
				 
		    // Step 6: Defining the Parameters of the vehicle used for the emission calculation (if not defined profile parameters are used)
		    // For detailed information refer to: 
		    // http://localhost:50030/xroute/com/ptvag/xserver/xroute/doc.html#com.ptvag.xserver.xroute.VehicleParameter
		    CountryInfoVehicleOptions countryInfoVehcileOptions = new CountryInfoVehicleOptions();  
		    VehicleOption vehType = new VehicleOption 
            {
                parameter = VehicleParameter.EMISSION_TECHNOLOGY,
                value = "EGR"
            };

            VehicleOption vehType0 = new VehicleOption 
            {
                parameter = VehicleParameter.EMISSION_CLASS,
                value = "EURO_5"
            };

            // Note: this is the official permitted weight of the vehicle 
		    // (see vehicle registration certificate)
		    // All three parameters are necessary in order to calculate correct emissions.
            VehicleOption vehType1 = new VehicleOption 
            {
                parameter = VehicleParameter.TOTAL_WEIGHT,
                value = "40000"
            };

            // Note: empty weight of chosen vehicle type 
            VehicleOption vehType2 = new VehicleOption 
            {
                parameter = VehicleParameter.EMPTY_WEIGHT,
                value = "12000"
            };

            // Note: load weight of chosen vehicle type
            VehicleOption vehType3 = new VehicleOption 
            {
                parameter = VehicleParameter.LOAD_WEIGHT,
                value = "5000"
            };

            VehicleOption vehType4 = new VehicleOption 
            {
                parameter = VehicleParameter.TYPE,
                value = "TRU"
            };

            VehicleOption vehType5 = new VehicleOption 
            {
                parameter = VehicleParameter.FUEL_TYPE,
                value = "DIESEL"
            };

		
		    VehicleOption[] options = { vehType, vehType0, vehType1, vehType2, vehType3, vehType4, vehType5 };
		    countryInfoVehcileOptions.wrappedOptions = options;
		
		    try {
			    // Step 7: calculate route
			    ExtendedRoute route = xRouteClient.calculateExtendedRoute(
						    waypointDescs, null, null, resultListOptions, countryInfoVehcileOptions, callerContext);
								
			    List<routeTableEntry> rows = new List<routeTableEntry>();
			
			    double x = 0;
			    double y = 0;
			    double carbondioxide = 0;
			    double carbonmonoxide = 0;
			    double nitrogenoxide = 0;
			    double distance = 0;
			
			    // Step 8: get the emission information per station
			    foreach (WayPoint station in route.route.wrappedStations){
			   
			        distance = station.accDist;			    
				    x = station.locationCoord.point.x;
                    y = station.locationCoord.point.y;
                    carbondioxide = station.emissions.carbonDioxide;
                    carbonmonoxide = station.emissions.carbonMonoxide;
                    nitrogenoxide = station.emissions.nitrogenOxides;
				
			        // create a new row in the table with the desired information				
                    routeTableEntry row = new routeTableEntry
                    {
                        x = x,
                        y = y,
                        dist = distance,
                        co2 = carbondioxide,
                        co = carbonmonoxide,
                        no = nitrogenoxide
                    };
			    }
                
                // Step 9: add items to table
                gridRoute.ItemsSource = rows;
		    }
		    catch(Exception e){
			    // TODO proper exception handling
		    }
	    }
    }

    public class routeTableEntry
    {
        public double x { get; set; }
        public double y { get; set; }
        public double dist { get; set; }
        public double co2 { get; set; }
        public double co { get; set; }
        public double no { get; set; }
    }
}
