﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithFuzzy
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Define array of points, points[0] = start, points[1] = destination, points[2] = via
            xserver.Point startPoint = new xserver.Point();
            startPoint.point = new PlainPoint { x = 980172, y = 6286080 };
            xserver.Point destPoint = new xserver.Point();
            destPoint.point = new PlainPoint { x = 989947, y = 6285085 };
            xserver.Point viaPoint = new xserver.Point();
            viaPoint.point = new PlainPoint { x = 990923, y = 6291255 };
            xserver.Point[] points = new xserver.Point[] { startPoint, destPoint, viaPoint };

            //Step 2: Run routing
            Route r = runRouting(points);
            
            //Step 3: Display result on the map
            if (r != null)
                mapRoute(r, points);
        }
        
        private Route runRouting(xserver.Point[] points) {
            
            // Step 1: Initialize client
            XRouteWSService xRouteClient = new XRouteWSService();
		
		    // Step 2: Define waypoints
		    // Note: german map is required
		
		    // start
		    WaypointDesc wpdStart = new WaypointDesc();
            wpdStart.wrappedCoords = new xserver.Point[] { points[0] };
		    wpdStart.linkType = LinkType.AUTO_LINKING;

		    // destination
		    WaypointDesc wpdDest = new WaypointDesc();
            wpdDest.wrappedCoords = new xserver.Point[] { points[1] };
		    wpdDest.linkType = LinkType.AUTO_LINKING;
			
		    // soft via-point
		    WaypointDesc wpdVia = new WaypointDesc();
            wpdVia.wrappedCoords = new xserver.Point[] { points[2] };
		
		    // set linking to fuzzy
		    wpdVia.linkType = LinkType.FUZZY_LINKING; 
		    wpdVia.fuzzyRadius = 1000;

            WaypointDesc[] waypointDescs = new WaypointDesc[] { wpdStart, wpdVia, wpdDest };
		    
            //Step 3: Set routing and result options
		    RoutingOption routingOption1 = new RoutingOption {
		        parameter = RoutingParameter.EXCLUDE_COUNTRIES,
		        value = "33"
            };
		    
		    RoutingOption[] routingOptions = { routingOption1 };

            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true,
                totalRectangle = true,
                detailLevel = DetailLevel.STANDARD
            };

		    try {
			    // Step 4: calculate route
                Route route = xRouteClient.calculateRoute(
					    waypointDescs, 
					    routingOptions, 
					    null,
					    resultListOptions,
                        null);

                return route;
			
		    }catch(Exception e){
			    // TODO proper exception handling
		    }
            return null;
	    }

        private void mapRoute(Route route, xserver.Point[] points) {
            // Step 1: Initialize client
            XMapWSService xMapClient = new XMapWSService();

            // Step 2: Create map with routes
            // Step 2.1: Use bounding box of route
            BoundingBox boundingBox = new BoundingBox();
            boundingBox.leftTop = route.totalRectangle.leftBottom;
            boundingBox.rightBottom = route.totalRectangle.rightTop;

            // Step 2.2: Get "polygon" from route
		    LineString[] lineStrings = new LineString[]{ route.polygon};

            // Step 2.3: Set line options ( color, stroke size )		 
		    LineOptions options =  new LineOptions();
            LinePartOptions linePartOptions = new LinePartOptions
            {
                color = new xserver.Color { red = 255, green = 0, blue = 0 },
                visible = true,
                width = -6
            };
		    options.mainLine = linePartOptions;
		    options.showFlags = true;
		    options.transparent = true;

		    Lines[] linesArray = new Lines[]{ new Lines { wrappedLines = lineStrings,options = options } };

            // Define bitmap for via-point
            Bitmap bitmap = new Bitmap
            {
                name = "location_rough.bmp",
                descr = "SoftVia",
                position = points[2]
            };
		
		    Bitmap[] bitmapArray = new Bitmap[]{ bitmap };
		
		    BitmapOptions bitmapOptions = new BitmapOptions();
		    bitmapOptions.transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 };
		    Bitmaps[] bitmaps = new Bitmaps[]{ new Bitmaps {wrappedBitmaps = bitmapArray, options = bitmapOptions } };

            // Step 2.4: put lines into a CustomLayer
            CustomLayer customLayer = new CustomLayer
            {
                name = "routeLayer",
                wrappedLines = linesArray,
                drawPriority = 500,
                centerObjects = true,
                objectInfos = ObjectInfoType.FULLGEOMETRYCLIPPED,
                visible = true,
                wrappedBitmaps = bitmaps
            };

            // Step 2.5: Create Image info and set map parameters
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 600,
                width = 800,
                imageParameter = ""
            };

            MapParams mapParams = new MapParams { showScale = false, useMiles = false, language = "x-ptv-DFT" };  

		    try {
			    // Step 3: generate map
                Map map = xMapClient.renderMapBoundingBox(
					    boundingBox, mapParams, imageInfo, new Layer[]{customLayer}, false, null);

			    // Step 4: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Step 5: Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
			
		    }catch(Exception e){
			    // TODO: proper exception handling
		    }
	    }
    }
}
