﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithIsochrones
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        static double x = 611798.0;
	    static double y = 4962511.0;

        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            Isochrone iso = runRouting();
            if (iso != null)
                runMapping(iso);
        }
        private static Isochrone runRouting()
        {
            // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();

            // Step 2: Create callerContextProperty and set profile truckfast.
            CallerContextProperty profile1 = new CallerContextProperty
            {
                key = "profile",
                value  = "truckfast"
            };
            CallerContextProperty profile2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_GEODECIMAL"
            };
            CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile1, profile2 };
            CallerContext callerContext = new CallerContext
            {
                wrappedProperties = callerContextProperties,
            };

            // Step 3: Define center location and add to class WaypointDesc.
            xserver.Point centerPoint = new xserver.Point();
            centerPoint.point = new PlainPoint { x = MainWindow.x, y = MainWindow.y } ;

            WaypointDesc wpdCenter = new WaypointDesc();
            wpdCenter.wrappedCoords = new xserver.Point[] { centerPoint };
            wpdCenter.linkType = LinkType.AUTO_LINKING;

            // Step 4.1: Create expansionDesc and set distance for expansion and horizon
            ExpansionDescription expansionDesc = new ExpansionDescription
            {
                expansionType = ExpansionType.EXP_DIST,
                wrappedHorizons = new int[] { 20000}
            };

            // Step 4.2: Define isochroneOptions and add expansion's description plus
            // detail
            IsochroneOptions isochroneOptions = new IsochroneOptions
            {
                expansionDesc = expansionDesc,
                isoDetail = IsochroneDetail.POLY_AND_SEGMENTS
            };

            try
            {
                // Step 5: Call method calculateIsochrones and return object isochrone
                Isochrone isochrone = xRouteClient.calculateIsochrones(wpdCenter, null, isochroneOptions, callerContext);
                return isochrone;
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
                return null;
            }
        }

        private void runMapping(Isochrone isochrone)
        {
            // Step 1: Instantiate xMap client
            XMapWSService xMapClient = new XMapWSService();

            // Step 2: Create callerContextProperty and set profile truckfast.
            CallerContextProperty profile1 = new CallerContextProperty
            {
                key = "profile",
                value = "truckfast"
            };
            CallerContextProperty profile2 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_GEODECIMAL"
            };
            CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile1, profile2 };
            CallerContext callerContext = new CallerContext
            {
                wrappedProperties = callerContextProperties,
            };

            // Step 3: Prepare polygon to draw the area which is assigned by the
            // isochrone.
            PlainPoint[] points = isochrone.wrappedIsochrones[0].polys.lineString.wrappedPoints;

            PlainLinearRing ring = new PlainLinearRing();
            ring.wrappedPoints = points;

            PlainPolygon plainPoly = new PlainPolygon();
            plainPoly.wrappedLinearRings = new PlainLinearRing[] { ring };

            xserver.Polygon polygon = new xserver.Polygon();
            polygon.polygon = plainPoly;

            // Step 4: Create referencePoint
            xserver.Point referencePoint = new xserver.Point();
            referencePoint.point = new PlainPoint { x = MainWindow.x, y = MainWindow.y };

            // Step 5.1: Create several objects as described below to realize geometry
            // layer.
            xserver.Geometry geometry = new xserver.Geometry();
            geometry.description = "Isochrones";
            // Use this method to return LineString. But take into account that only the
            // border of the isochrones will be drawn.
            // geometry.geometry = isochrone.wrappedIsochrones[0].polys;
            geometry.geometry = polygon;
            geometry.id = 1;
            geometry.referencePoint = referencePoint;
            xserver.Geometry[] geometries = new xserver.Geometry[] { geometry };

            // Step 5.2: set how the objects will be drawn
            GeometryOption geometryCenterObjects = new GeometryOption
            {
                option = GeometryOptions.AUTOCENTEROBJECTS,
                value = "1"
            };

            String fillColor = rgbToColorString(50, 50, 255);
            GeometryOption geometryFillColor = new GeometryOption
            {
                option = GeometryOptions.FILLCOLOR,
                value = fillColor
            };

            String lineColor = rgbToColorString(255, 0, 0);
            GeometryOption geometryLineColor = new GeometryOption
            {
                option = GeometryOptions.LINECOLOR,
                value = lineColor
            };

            GeometryOption geometryLineWidth = new GeometryOption
            {
                option = GeometryOptions.LINEWIDTH,
                value = "6"
            };

            GeometryOption[] options = new GeometryOption[] { geometryCenterObjects, geometryFillColor, geometryLineColor,
				geometryLineWidth };

            Geometries isochrones = new Geometries { wrappedGeometries = geometries, wrappedOptions = options } ;

            GeometryLayer geometryLayer = new GeometryLayer
            {
                name = "Isochrones",
                drawPriority = 100,
                visible = true,
                objectInfos = ObjectInfoType.NONE,
                wrappedGeometries = new Geometries[] { isochrones }
            };

            // Step 5.3: Create new object layers and put customLayer into it.
            Layer[] layers = new Layer[] { geometryLayer };

            // Step 6: Define mapSection and set center.
            MapSection mapSection = new MapSection();
            mapSection.center = referencePoint;

            // Step 7: Define imageInfo and define height and width of the image.
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 600,
                width = 800,
                imageParameter = ""
            };

            // Step 8: Use mapParams for scale and distance unit
            MapParams mapParams = new MapParams { showScale = false, useMiles = false, language = "x-ptv-DFT" };

            try
            {
                // Step 9: call renderMap and return object map.
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, false, callerContext);

                // Step 8: Retrieve image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Step 10: Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
            }
        }

        private String rgbToColorString(int red, int green, int blue)
        {
            int color = (blue << 16) + (green << 8) + (red);
            return "" + color;
        }
    }
}
