﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using RoutingWithMatrixInfo.XRoute;

namespace RoutingWithMatrixInfo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample() {
    	     // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();

            // Step 2: Define starting points
            WaypointDesc[] startWPs = new WaypointDesc[2];
            // Rue Principale 27
            // L-5240 Sandweiler
            XRoute.Point point1 = new XRoute.Point();
		    point1.point = new PlainPoint { x = 691495.7, y = 6372987.6 };
		
            startWPs[0] = new WaypointDesc();
            startWPs[0].wrappedCoords = new XRoute.Point[] { point1 };
            // Steil 2
            // L-9651 Eschweiler
            XRoute.Point point2 = new XRoute.Point();
            point2.point = new PlainPoint { x = 661614.3, y = 6438775.4 };
            startWPs[1] = new WaypointDesc();
            startWPs[1].wrappedCoords = new XRoute.Point[] { point2 };

            // Step 3: Define destination points
            WaypointDesc[] destWPs = new WaypointDesc[3];
            // Rue du Nord 16
            // L-8806 Rambrouch
            XRoute.Point point3 = new XRoute.Point();
            point3.point = new PlainPoint { x = 650023.3, y = 6410109.7 };
            destWPs[0] = new WaypointDesc();
            destWPs[0].wrappedCoords = new XRoute.Point[] { point3 };
            // Burrewee 3
            // L-9760 Wilwerwiltz
            XRoute.Point point4 = new XRoute.Point();
            point4.point = new PlainPoint { x = 668520.6, y = 6436053.1 };
            destWPs[1] = new WaypointDesc();
            destWPs[1].wrappedCoords = new XRoute.Point[] { point4 };
            // Rue des Violettes 10
            // L-4940 Bascharage
            XRoute.Point point5 = new XRoute.Point();
            point5.point = new PlainPoint { x = 657647.9, y = 6365485.9 };
            destWPs[2] = new WaypointDesc();
            destWPs[2].wrappedCoords = new XRoute.Point[] { point5 };

            try {
                // Step 4: calculate matrix
                MatrixInfo matrixInfo = xRouteClient.calculateMatrixInfo(startWPs, destWPs, null, null, null);
                int index = 0;
                // Step 5: print out calculated information
                List<matrixEntry> rows = new List<matrixEntry>();
                foreach ( RouteInfo routeInfo in matrixInfo.wrappedMatrixInfo) {
            	    matrixEntry row = new matrixEntry 
                    {
                        id = index,
                        cost = routeInfo.cost,
                        dist = routeInfo.distance / 1000.0,
                        time = routeInfo.time / 60.0,
                    };
                    rows.Add(row);
                    index++;
                }
                gridRoute.ItemsSource = rows;

            }catch(Exception e){
                // TODO proper exception handling
            }
        }
    }
    public class matrixEntry
    {
        public int id { get; set; }
        public double cost { get; set; }
        public double dist { get; set; }
        public double time { get; set; }
    }
}
