﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using RoutingWithToll.XRoute;

namespace RoutingWithToll
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // Step 1: Instantiate xRoute client
            XRouteWSService xRouteClient = new XRouteWSService();

		    // Step 2.1: Create callerContextProperty and set profile truckfast.
		    CallerContextProperty profile = new CallerContextProperty();
		    profile.key = "Profile";
		    profile.value = "truckfast";

		    // Step 2.2: Set properties into callerContextProperties (type is defined as
		    // an array)
		    CallerContextProperty[] callerContextProperties = new CallerContextProperty[] { profile };

		    // Step 2.3: Define callerContext, set properties and overwrite default
		    // class in xRoute client
		    CallerContext callerContext = new CallerContext();
		    callerContext.wrappedProperties = callerContextProperties;

		    // Step 3: Add way-points for routing
		    // Note: PTV Europe City Map Premium is required and additional toll data
		    // has to be licensed.
		    double[][] points = new double[][] { new double [] { 262175.0, 6242590.0 }, // Paris
				    new double [] { 933441.0, 6268969.0 }, // Karlsruhe
				    new double [] { 1820467.0, 6134685.0 }, // Wien
				    new double [] { 1267389.0, 5978788.0 }, // Innsbruck
				    new double [] { 1262552.0, 5853474.0 } // Bolzano
		    };

		    WaypointDesc[] waypointDescs = new WaypointDesc[points.Length];
		    for (int i = 0; i < points.Length; i++) {
			    XRoute.Point pt = new XRoute.Point();
			    pt.point = new PlainPoint { x = points[i][0], y = points[i][1] } ;

			    waypointDescs[i] = new WaypointDesc();
			    waypointDescs[i].wrappedCoords = new XRoute.Point[] { pt };
			    waypointDescs[i].linkType = LinkType.AUTO_LINKING;
		    }

		    // Step 4: Specify the ResultListOptions. They provide the information, e.g.
		    // how
		    // detailed the itinerary should be listed.
            ResultListOptions resultListOptions = new ResultListOptions
            {
		        manoeuvres = true,
		        texts = true,
		        segments = true,
		        detailLevel = DetailLevel.STANDARD
            };

		    // Step 5.1: Define countryInfoVehicleOptions and vehicleOption. Set the
		    // relevant attributes.
		    CountryInfoVehicleOptions countryInfoVehicleOptions = new CountryInfoVehicleOptions
            {
		        allEuro = true,
		        detailedTollCosts = true
            };

		    VehicleOption vehicleOption = new VehicleOption
            {
                parameter = VehicleParameter.TYPE,
		        value = "TRU"
            };
		    

		    // Step 5.2: Put the vehicleOption into the array VehicleOption[]
		    VehicleOption[] vehicleOptions = { vehicleOption };

		    // Step 5.3: Set option "vehicleOptions" into countryInfoVehicleOptions
		    countryInfoVehicleOptions.wrappedOptions = vehicleOptions;

		    try {
			    // Step 6: Return route including the toll information
			    ExtendedRoute route = xRouteClient.calculateExtendedRoute(waypointDescs, null, null, resultListOptions,
					    countryInfoVehicleOptions, callerContext);

			    // Step 7: Plot overview of toll prices and distances
			    RoutingTableEntry[] rows = new RoutingTableEntry[route.wrappedCountryInfos.Length];
                int id = 0;
			    foreach (CountryInfo countryInfo in route.wrappedCountryInfos) {
				    int totalTollPrice = 0;
				    int totalTollDistance = 0;

				    foreach (TollCostInfo tollCostInfo in countryInfo.wrappedTollCostInfos) {
					    totalTollPrice += tollCostInfo.tollPrice;
					    totalTollDistance += tollCostInfo.tollDistance;
				    }

                    RoutingTableEntry row = new RoutingTableEntry
                    {
                        code = countryInfo.iuCode,
                        dist = countryInfo.partRouteInfo.distance / 1000.0,
                        tollDist = totalTollDistance / 1000.0,
                        tollPrice = totalTollPrice / 1000.0,
                    };
                    rows[id] = row;
                    id++;
			    }

                // Step 8: Output the data in the grid
                gridRoute.ItemsSource = rows;
		    } catch (Exception e)
            {
                //TODO: proper exception handling
            }
		    
        }
    }

    public class RoutingTableEntry
    {
        public int code { get; set; }
        public double dist { get; set; }
        public double tollDist { get; set; }
        public double tollPrice { get; set; }
    }
}
