﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace RoutingWithTruckAttributes
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private static CallerContext callerContext { get; set; }

        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // calculate route
            Route r = calculateRoute();
            
            // display route
            if (r != null)
                displayMapOfRoute(r);
        }

        private Route calculateRoute() 
        {
		    // Step 1: Initialize client
		    XRouteWSService xRouteClient = new XRouteWSService();

		    // Step 2.1: Add attribute "Profile" into class CallerContext
		    CallerContextProperty profile = new CallerContextProperty {key = "Profile", value = "truckfast" } ;
		    callerContext = new CallerContext();
		    callerContext.wrappedProperties = new CallerContextProperty[] { profile };

		    // Step 3.1: Enable RoadEditor
		    RoutingOption enableRE = new RoutingOption { parameter = RoutingParameter.ENABLE_ROADEDITOR, value = "true" };

		    // Step 3.2: Define reLayer 
		    // Demo version: maps.path=data/map is enabled.
		    // re/truckattributes folder is already available --> see folder data/map/re
		    RoutingOption reLayer = new RoutingOption { parameter = RoutingParameter.ROADEDITOR_LAYERNAME, value = "truckattributes" };

		    // Step 3.3: Add additional options
		    RoutingOption reOptions = new RoutingOption { parameter = RoutingParameter.ROADEDITOR_ADDITIONAL_OPTIONS, value = "MAX_HEIGHT=401" };
		    RoutingOption[] routingOptions = new RoutingOption[] { enableRE, reLayer, reOptions };
		
		    // Step 4: Specify the ResultListOptions
            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true, 
                totalRectangle = true,
                detailLevel = DetailLevel.STANDARD,
                segments = true,
                segmentAttributes = true
            };

		    // Step 5: Get array waypointsDesc[]
		    WaypointDesc[] waypointDescs = getWaypoints();

		    try {
			    // Step 6: Call method calculateRoute
			    Route route = xRouteClient.calculateRoute(waypointDescs, routingOptions, null, resultListOptions, callerContext);

                return route;
		    } catch (Exception e) {
			    // TODO: proper exception handling
		    }
            return null;
	    }

	    private void displayMapOfRoute(Route route)
        {
            // Step 1: Initialize client
            XMapWSService xMapClient = new XMapWSService();
		    
            // Step 2: Define imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

		    // Step 3: Add mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = true;
            mapParams.language = "x-ptv-DFT"; //optional
		    bool includeImageInResponse = false;

		    // Step 4: Create a line from the given route
		    LineString[] lineStrings = new LineString[] { route.polygon };
		    Lines[] lines = new Lines[1];
		    LineOptions options = new LineOptions();
            LinePartOptions linePartOpt = new LinePartOptions
            {
                color = new xserver.Color { red = 0, blue = 255, green = 0 },
                visible = true,
                width = -12
            };
		    options.mainLine = linePartOpt;

            lines[0] = new Lines { wrappedLines = lineStrings, options = options };

		    // Step 5: Define a custom layer that contains the line(s)
            CustomLayer customLayer = new CustomLayer 
            {
                visible = true,
                drawPriority = 100, // this is PRIORROUTE
                wrappedLines = lines,
                objectInfos = ObjectInfoType.NONE,
                centerObjects = true
            };

		    // Step 6: Create reLayer
		    // Demo version: maps.path=data/map is enabled.
		    // re/truckattributes folder is already available --> see folder data/map/re
            RoadEditorLayer reLayer = new RoadEditorLayer 
            {
                name = "truckattributes",
		        visible = true,
		        objectInfos = ObjectInfoType.NONE
            };
		    
		    // Layer[] layers = new Layer[]{ reLayer};

		    Layer[] layers = new Layer[] { customLayer, reLayer };

		    // Step 7: Set the bounding box
		    BoundingBox boundingBox = new BoundingBox 
            {
                leftTop = route.totalRectangle.leftBottom,
                rightBottom = route.totalRectangle.rightTop,
            };

            try
            {
                // Step 8: Call method renderMapBoundingBox
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, callerContext);
                // Step 10: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
	    }

	    private WaypointDesc[] getWaypoints() {
		    // points = list of pairs of (x,y) coordinates (in order of route)
            double[][] points = new double[][] { new double[] { 681949.0, 6370040.0 }, new double[] { 681144.0, 6365010.0 } };

		    WaypointDesc[] waypointDescs = new WaypointDesc[points.Length];
		    for (int i = 0; i < points.Length; i++) {
                xserver.Point pt = new xserver.Point();
                pt.point = new PlainPoint { x = points[i][0], y = points[i][1]};

                waypointDescs[i] = new WaypointDesc
                {
                    wrappedCoords = new xserver.Point[] { pt },
                    linkType = LinkType.NEXT_SEGMENT
                };
		    }
		    return waypointDescs;
	    }
    }
}
