﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace TimeZones
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private DateTime startTime;
        
        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        private void runSample()
        {
            // calculate route
            Route r = calculateRoute();

            // display route
            if (r != null)
                displayMapOfRoute(r);
        }

        private Route calculateRoute()
        {
            // Step 1: Initialize client
            XRouteWSService xRouteClient = new XRouteWSService();

            // Step 2.1: Initialize CallerContext and add properties
            CallerContext callerContext = new CallerContext();
            CallerContextProperty profile = new CallerContextProperty { key = "Profile", value = "truckfast" };
            CallerContextProperty format = new CallerContextProperty { key = "CoordFormat", value = "PTV_GEODECIMAL" };

            // Step 2.2: If TimeZones are used add an XML Snippet to the CallerContext
            CallerContextProperty useTimeZones = new CallerContextProperty();
            if (useTime.IsChecked == true)
            {
                String xml = @"<?xml version=""1.0"" encoding=""UTF-8"" standalone=""yes""?>
                            <Profile xmlns:ns2=""http://www.ptv.de/com/ptvag/xserver/engines/xpoidbaccess"" >
                                <FeatureLayer majorVersion=""1"" minorVersion=""0"">
                                    <Themes>
                                        <Theme id=""PTV_TimeZones"" enabled=""false"">
                                        </Theme>
                                    </Themes>
                                </FeatureLayer>
                                <Routing majorVersion=""2"" minorVersion=""0"">
                                    <Course>
                                        <AdditionalDataRules enabled=""true""/>
                                    </Course>
                                </Routing>
                            </Profile>";
                useTimeZones = new CallerContextProperty();
                useTimeZones.key = "ProfileXMLSnippet";
                useTimeZones.value = xml;
            }

            callerContext.wrappedProperties = new CallerContextProperty[] { profile, useTimeZones, format };

            // Step 3: Specify the ResultListOptions
            ResultListOptions resultListOptions = new ResultListOptions
            {
                polygon = true,
                totalRectangle = true,
                detailLevel = DetailLevel.STANDARD,
                segments = true,
                segmentAttributes = true,
                utcOffsets = true,
                utcOffsetsSpecified = true
            };

            // Step 4: Get array waypointsDesc[]
            WaypointDesc[] waypointDescs = getWaypoints();

            // Step 5: Add a starttime as a routing option, this is relevant in order to use historic traffic data
            // Example for the date format: 2014-02-05T08:00:00+02:00 (xsd:datetime format)
            RoutingOption starttime = new RoutingOption
            {
                parameter = RoutingParameter.START_TIME,
                value = getTimeString()
            };

            RoutingOption[] routingOptions = { starttime };

            try
            {
                // Step 5: Call method calculateRoute
                Route route = xRouteClient.calculateRoute(waypointDescs, routingOptions, null, resultListOptions, callerContext);
                calculateETA(route);

                return route;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
            return null;
        }

        private void displayMapOfRoute(Route route)
        {
            // Step 1: Initialize client
            XMapWSService xMapClient = new XMapWSService();

            // Step 2: Initialize CallerContext and add properties
            CallerContext callerContext = new CallerContext();
            CallerContextProperty format = new CallerContextProperty { key = "CoordFormat", value = "PTV_GEODECIMAL" };
            callerContext.wrappedProperties = new CallerContextProperty[] { format };

            // Step 3: Define imageInfo
            ImageInfo imageInfo = new ImageInfo();
            imageInfo.format = ImageFileFormat.GIF;
            imageInfo.height = 600;
            imageInfo.width = 800;
            imageInfo.imageParameter = "";

            // Step 4: Add mapParams
            MapParams mapParams = new MapParams();
            mapParams.showScale = true;
            mapParams.useMiles = false;
            mapParams.language = "x-ptv-DFT"; //optional
            bool includeImageInResponse = false;

            // Step 5: Create a line from the given route
            LineString[] lineStrings = new LineString[] { route.polygon };
            Lines[] lines = new Lines[1];
            LineOptions options = new LineOptions();
            LinePartOptions linePartOpt = new LinePartOptions
            {
                color = new xserver.Color { red = 0, blue = 255, green = 0 },
                visible = true,
                width = -12
            };
            options.mainLine = linePartOpt;

            lines[0] = new Lines { wrappedLines = lineStrings, options = options };

            // Step 6: Define a custom layer that contains the line(s)
            CustomLayer customLayer = new CustomLayer
            {
                visible = true,
                drawPriority = 100, // this is PRIORROUTE
                wrappedLines = lines,
                objectInfos = ObjectInfoType.NONE,
                centerObjects = true
            };

            Layer[] layers = new Layer[] { };
            layers = new Layer[] { customLayer };

            // Step 7: Set the bounding box
            BoundingBox boundingBox = new BoundingBox
            {
                leftTop = route.totalRectangle.leftBottom,
                rightBottom = route.totalRectangle.rightTop,
            };

            try
            {
                // Step 8: Call method renderMapBoundingBox
                Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams, imageInfo, layers, includeImageInResponse, callerContext);
                // Step 9: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                BitmapImage bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                img1.Source = bitmapImage;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
            }
        }

        private WaypointDesc[] getWaypoints()
        {
            // points = list of pairs of (x,y) coordinates (in order of route)
            double[][] points = new double[][] { new double[] { 1340495, 5252000 }, new double[] { -913933, 3872225 } };

            WaypointDesc[] waypointDescs = new WaypointDesc[points.Length];
            for (int i = 0; i < points.Length; i++)
            {
                xserver.Point pt = new xserver.Point();
                pt.point = new PlainPoint { x = points[i][0], y = points[i][1] };

                waypointDescs[i] = new WaypointDesc
                {
                    wrappedCoords = new xserver.Point[] { pt },
                    linkType = LinkType.NEXT_SEGMENT
                };
            }
            return waypointDescs;
        }

        private void calculateETA(Route r)
        {
            WayPoint start = r.wrappedStations[0];
            WayPoint end = r.wrappedStations[r.wrappedStations.Length - 1];
            double hours = r.info.time / 3600;
            double minutes = r.info.time / 60;
            int actHours = (int)Math.Floor(hours);
            actHours += Convert.ToInt32(startTime.Hour);
            int actMinutes = (int)Math.Floor(minutes % 60);
            actMinutes += Convert.ToInt32(startTime.Minute);
            actHours += actMinutes / 60;
            actMinutes = actMinutes % 60;
            actHours = actHours % 24;
            if (useTime.IsChecked == true)
            {
                actHours += (end.utcOffset - start.utcOffset) / 60;
                actMinutes += (end.utcOffset - start.utcOffset) % 60;
            }
            ETABox.Text = actHours.ToString() + ":" + actMinutes.ToString();
        }

        private String getTimeString()
        {
            String time;
            startTime = DateTime.Now;
            time = startTime.ToString("s");
            time += "+02:00";
            return time;
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            runSample();
        }
    }
}
