﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using xserver;

namespace Geometry
{
    /**
     * Used to create a so called characteristic shape (Chi-Shape) of a set of points 
     * (http://www.geosensor.net/phpws/index.php?module=pagemaster&PAGE_user_op=view_printable&PAGE_id=13&lay_quiet=1)
     * When the calculation is finished, the externalEdges represent the Chi-Shape
     * 
     * A Chi-Shape is formed, by removing external edges from the delaunay triangulation of a set of points, based on their length.
     * This leads to the removal of very long edges, which obscure the real shape of a point set. (http://www.geosensor.net/phpws/images/pagemaster/charhulls_big.jpg)
    **/
    class ChiTriangulation
    {
        public List<Edge> externalEdges { get; private set; }
        public Dictionary<int, Geometry.Point> externalPoints { get; private set; }
        public Dictionary<int, Geometry.Triangle> triangles { get; private set; }
        public Dictionary<int, Edge> edges { get; private set; }

        public ChiTriangulation()
        {
            externalEdges = new List<Edge>();
            externalPoints = new Dictionary<int, Point>();
            triangles = new Dictionary<int, Triangle>();
            edges = new Dictionary<int, Edge>();
        }
    }
}
