﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using xserver;

namespace Geometry
{
    /**
     * Represents a circle, given by a center and a radius
    **/
    class Circle
    {
        public PlainPoint center { get; private set; }
        public double radius { get; private set;  }

        private double minX { get; set; }
        private double maxX { get; set; }
        private double minY { get; set; }
        private double maxY { get; set; }

        public Circle(PlainPoint center, double radius)
        {
            this.center = center;
            this.radius = radius + 0.000001;
            
            maxX = center.x + this.radius;
            minX = center.x - this.radius;
            maxY = center.y + this.radius;
            minY = center.y - this.radius;
        }

        // Tests, if a point is inside a circle
        public bool isInside(Point p)
        {
            if (insideBoundingBox(p))
            {
                double dist = Math.Sqrt(Math.Pow(p.pt.x - center.x, 2) + Math.Pow(p.pt.y - center.y, 2));
                if (dist <= radius)
                    return true;
                else
                    return false;
            }
            else
            {
                return false;
            }
        }

        // Tests, if a point is inside the bounding box of the circle
        private bool insideBoundingBox(Point p)
        {
            if ((p.pt.x <= maxX) && (p.pt.x >= minX) && (p.pt.y <= maxY) && (p.pt.y >= minY))
                return true;
            else
                return false;
        }
    }
}
