﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using xserver;

namespace SequencingWithPriorities
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        public static Dictionary<int, double[]> numbers;

        public MainWindow()
        {
            InitializeComponent();
            runSample();
        }

        public void runSample()
        {
            SequencingPlan plan = runSequencingSample();

            if (plan != null)
            {
                BitmapImage image = new BitmapImage();
                runMappingSample(plan, ref image);

                img1.Source = image;

                List<TourOrderTableEntry> tourOrders = new List<TourOrderTableEntry>();
                List<TourEventTableEntry> tourEvents = new List<TourEventTableEntry>();
                List<TourPointTableEntry> tourPoints = new List<TourPointTableEntry>();

                runSequencingResultOrderID(plan, ref tourOrders);
                runSequencingResultTourEvents(plan, ref tourEvents);
                runSequencingResultTourPoints(plan, ref tourPoints);

                gridOrders.ItemsSource = tourOrders;
                gridEvents.ItemsSource = tourEvents;
                gridPoints.ItemsSource = tourPoints;
            }
        }

        private BitmapImage runMappingSample(SequencingPlan plan, ref BitmapImage bitmapImage)
        {
            // Step 1: Instantiate xMap client.
            XMapWSService xMapClient = new XMapWSService();

            // Step 2: Create object MapSection
            MapSection mapSection = new MapSection();
            xserver.Point point = new xserver.Point();
            double[] xy = numbers[0];
            point.point = new PlainPoint { x = xy[0], y = xy[1] };
            mapSection.center = point;
            mapSection.scale = 500;

            // Step 3: Define an array of lines
            int numberOfDestinationPoints = 0;
            foreach (TourEvent tourEvents in plan.result.wrappedTourEvents)
            {
                int tourPointID = tourEvents.logisticPointId;
                if (tourPointID != -1)
                {
                    numberOfDestinationPoints++;
                }
            }
            PlainPoint[] destinationPlainPoints = new PlainPoint[numberOfDestinationPoints];
            int[] orderID = new int[numberOfDestinationPoints];

            int index = 0;
            foreach (TourEvent tourEvents in plan.result.wrappedTourEvents)
            {
                int tourPointID = tourEvents.logisticPointId;

                if (tourPointID != -1)
                {
                    double[] loc = numbers[tourPointID];
                    orderID[index] = tourPointID;
                    // System.out.println("Locations: " + tourPointID + "  " + loc[0] + "  "
                    // + loc[1]);
                    PlainPoint plainPoint = new PlainPoint();
                    plainPoint.x = loc[0];
                    plainPoint.y = loc[1];
                    destinationPlainPoints[index] = plainPoint;
                    index++;
                }
            }

            Lines[] lines = new Lines[numberOfDestinationPoints];
            int[][] colorValues = new int[][] {
				    // {red, green, blue}
				    new int []{ 0, 0, 255 }, new int []{ 0, 0, 255 }, 
                    new int []{ 0, 255, 0 }, new int []{ 238, 134, 28 }, 
                    new int []{ 139, 123, 108 }, new int []{ 79, 79, 47 },
				    new int []{ 131, 139, 131 }, new int []{ 0, 290, 139 } };

            for (int i = 0; i < numberOfDestinationPoints - 1; i++)
            {
                // Step 3.1: Create plainLineString
                PlainLineString plainLineString = new PlainLineString();
                plainLineString.wrappedPoints = new PlainPoint[] { destinationPlainPoints[i], destinationPlainPoints[i + 1] };
                // Step 3.2: Set object into lineStrings[0]. Note: object lineStrings has
                // to be reinitialized in every loop.
                LineString[] lineStrings = new LineString[1];
                lineStrings[0] = new LineString();
                lineStrings[0].lineString = plainLineString;
                // Step 3.3: Create new lines[i] and fill array
                lines[i] = new Lines();
                lines[i].wrappedLines = lineStrings;
                // Step 3.4: Set lineOptions transparency
                LineOptions lineOptions = new LineOptions();
                lineOptions.transparent = true;
                // Step 3.5: Set linePartOptions visible, define width of line and
                // specific color
                LinePartOptions linePartOptions = new LinePartOptions
                {
                    visible = true,
                    width = 5,
                };
                linePartOptions.color = new xserver.Color { red = colorValues[0][0], green = colorValues[0][1], blue = colorValues[0][2] };
                // Step 3.6: Set main line
                lineOptions.mainLine = linePartOptions;
                // Step 3.7: Set all options for specific line
                lines[i].options = lineOptions;
            }

            // Step 4: Generate a new object named bitmap as type of Bitmap[]
            Bitmap[] bitmap = new Bitmap[numberOfDestinationPoints];
            for (int i = 0; i < numberOfDestinationPoints - 1; i++)
            {
                xserver.Point bitmapPointDestination = new xserver.Point();

                if (i == 0)
                {
                    bitmapPointDestination.point = destinationPlainPoints[i];
                    bitmap[i] = new Bitmap
                    {
                        name = "flaggreen.bmp",
                        position = bitmapPointDestination,
                        descr = "Start and Destination"
                    };
                }
                else
                {
                    bitmapPointDestination.point = destinationPlainPoints[i];
                    bitmap[i] = new Bitmap
                    {
                        name = "flagred.bmp",
                        position = bitmapPointDestination,
                        descr = "Tourstop"
                    };
                }
            }

            // Step 5: Create bitmapOptions and set color for transparency
            BitmapOptions bitmapOptions = new BitmapOptions();
            bitmapOptions.transparencyColor = new xserver.Color { red = 255, green = 255, blue = 255 };

            // Step 6: Create object bitmaps and put bitmap and bitmapOptions into its
            // array
            Bitmaps[] bitmaps = new Bitmaps[1];
            bitmaps[0] = new Bitmaps();
            bitmaps[0].wrappedBitmaps = bitmap;
            bitmaps[0].options = bitmapOptions;

            Text[] textTourpoints = new Text[numberOfDestinationPoints - 1];
            for (int i = 0; i < numberOfDestinationPoints - 1; i++)
            {
                xserver.Point pointText = new xserver.Point();
                pointText.point = destinationPlainPoints[i];

                Text text = new Text();
                text.position = pointText;
                text.text = "" + orderID[i];

                textTourpoints[i] = text;
            }

            // Step 7: Set text options
            Font font = new Font
            {
                name = "Arial",
                size = 14,
                bold = true
            };

            xserver.TextOptions textOptions = new xserver.TextOptions
            {
                font = font,
                textColor = new xserver.Color { red = 0, green = 0, blue = 0 },
                fillBg = true,
                bgColor = new xserver.Color { red = 255, green = 255, blue = 255 },
                showFrame = true,
                frameColor = new xserver.Color { red = 0, green = 0, blue = 255 },
                alignment = xserver.TextAlignment.CENTER,
                pixelX = 40,
                pixelY = -5,
                relX = 0,
                relY = 0
            };

            // Step 8: If you want to style each text object differently
            // you should create an object "Texts" for each text.
            // This allows you to use different text options.
            Texts texts = new Texts();
            texts.options = textOptions;
            texts.wrappedTexts = textTourpoints;

            // Step 9: Define customLayer and set the relevant attributes
            CustomLayer customLayer = new CustomLayer
            {
                drawPriority = 25000,
                centerObjects = true,
                name = "Text layer",
                objectInfos = ObjectInfoType.REFERENCEPOINT,
                visible = true,
                wrappedBitmaps = bitmaps,
                wrappedLines = lines,
                wrappedTexts = new Texts[] { texts }
            };

            // Step 10: Add customLayer into layers
            Layer[] layers = new Layer[] { customLayer };

            // Step 11: Create object ImageInfo
            ImageInfo imageInfo = new ImageInfo
            {
                format = ImageFileFormat.PNG,
                height = 530,
                width = 787
            };

            // Step 12: Create object MapParams
            MapParams mapParams = new MapParams
            {
                showScale = true,
                useMiles = false
            }; ;

            // Step 13: Set attribute includeImageInResponse
            bool includeImageInResponse = false;
            try
            {

                // Step 14: Call renderMapBoundingBox and return object map.
                Map map = xMapClient.renderMap(mapSection, mapParams, imageInfo, layers, includeImageInResponse, null);

                // Step 10: retrieve and display the image
                String imageUrl = "http://" + map.image.url;
                // Create BitmapImage
                bitmapImage = new BitmapImage();

                // Display BitmapImage
                bitmapImage.BeginInit();
                bitmapImage.UriSource = new Uri(imageUrl);
                bitmapImage.EndInit();

                return bitmapImage;
            }
            catch (Exception e)
            {
                //TODO: proper exception handling
                return null;
            }
        }

        private List<TourEventTableEntry> runSequencingResultTourEvents(SequencingPlan plan, ref List<TourEventTableEntry> TourEventTable)
        {
            // Get tour events
            foreach (TourEvent tourEvents in plan.result.wrappedTourEvents)
            {
                TourEventTableEntry row = new TourEventTableEntry();
                int id = tourEvents.logisticPointId;
                if (id == -1)
                {
                    row.ID = "";
                }
                else
                {
                    row.ID = id.ToString();
                }

                int start = tourEvents.startTime;
                row.start = second2hours(start);

                int period = tourEvents.period;
                row.period = second2hours(period);

                row.type = tourEvents.wrappedDescriptions[0];
                TourEventTable.Add(row);
            }

            return TourEventTable;
        }

        private static List<TourOrderTableEntry> runSequencingResultOrderID(SequencingPlan plan, ref List<TourOrderTableEntry> TourOrderTable)
        {
            int scheduledOrderId = plan.result.wrappedScheduledOrderIds.Length;
            int unscheduledOrderId = 0;
            bool available = false;

            if (plan.result.wrappedUnscheduledOrderIds != null)
            {
                unscheduledOrderId = plan.result.wrappedUnscheduledOrderIds.Length;
                available = true;
            }

            // Get scheduled order Id
            foreach (int orderID in plan.result.wrappedScheduledOrderIds)
            {
                TourOrderTableEntry row = new TourOrderTableEntry();
                row.type = "scheduled";
                row.ID = orderID;
                TourOrderTable.Add(row);
            }

            if (available)
            {
                foreach (int orderID in plan.result.wrappedUnscheduledOrderIds)
                {
                    TourOrderTableEntry row = new TourOrderTableEntry();
                    row.type = "unscheduled";
                    row.ID = orderID;
                    TourOrderTable.Add(row);
                }
            }

            return TourOrderTable;
        }

        private List<TourPointTableEntry> runSequencingResultTourPoints(SequencingPlan plan, ref List<TourPointTableEntry> TourPointTable)
        {
            // Get tour points
            foreach (SequencingTourPoint tourPoint in plan.tour.wrappedTourPoints)
            {
                String arrival = second2hours(tourPoint.result.arrivalTime);
                String driving = second2hours(tourPoint.result.drivingPeriod);
                TourPointTableEntry row = new TourPointTableEntry
                {
                    ID = tourPoint.id,
                    desc = tourPoint.description,
                    dist = tourPoint.result.drivingDistance / 1000.0,
                    arrival = arrival,
                    driving = driving,
                    type = tourPoint.type
                };
                TourPointTable.Add(row);
            }
            return TourPointTable;
        }

        private String second2hours(int second)
        {
            String time = "";
            String hours = "";
            String minutes = "";
            String seconds = "";

            int h = (second / 3600);
            if (h < 10)
            {
                hours = "0" + h;
            }
            else
            {
                hours = "" + h;
            }

            int rest1 = (second % 3600);
            int min = (rest1 / 60);
            if (min < 10)
            {
                minutes = "0" + min;
            }
            else
            {
                minutes = "" + min;
            }

            int sec = (rest1 % 60);
            if (sec < 10)
            {
                seconds = "0" + sec;
            }
            else
            {
                seconds = "" + sec;
            }

            time = (hours + ":" + minutes + ":" + seconds);
            return time;
        }

        private SequencingPlan runSequencingSample()
        {
            numbers = new Dictionary<int, double[]>();
            // Step 1.0: Define locations
            double[] xy1010 = new double[2];
            xy1010[0] = 675000.0;
            xy1010[1] = 6380000.0;
            numbers.Add(1010, xy1010);
            double[] xy1020 = new double[2];
            xy1020[0] = 671006.0;
            xy1020[1] = 6382294.0;
            numbers.Add(1020, xy1020);
            double[] xy1030 = new double[2];
            xy1030[0] = 682676.0;
            xy1030[1] = 6388154.0;
            numbers.Add(1030, xy1030);
            double[] xy1041 = new double[2];
            xy1041[0] = 681000.0;
            xy1041[1] = 6391000.0;
            numbers.Add(1041, xy1041);
            double[] xy1042 = new double[2];
            xy1042[0] = 682000.0;
            xy1042[1] = 6382000.0;
            numbers.Add(1042, xy1042);
            double[] xy201 = new double[2];
            xy201[0] = 681812.0;
            xy201[1] = 6375071.0;
            numbers.Add(201, xy201);
            // Note: Center of map tile
            double[] xyCenter = new double[2];
            xyCenter[0] = 685407.0;
            xyCenter[1] = 6372537.0;
            numbers.Add(0, xyCenter);
            // Step 1.1: Instantiate xTour client for sequencing
            XTourWSService xTourClient = new XTourWSService();
            // Step 1.2: Set a new CallerContext
            CallerContext cxt = new CallerContext();
            CallerContextProperty ccp1 = new CallerContextProperty
            {
                key = "CoordFormat",
                value = "PTV_MERCATOR",
            };
            CallerContextProperty ccp2 = new CallerContextProperty
            {
                key = "ResponseGeometry",
                value = "WKT",
            };
            CallerContextProperty ccp3 = new CallerContextProperty
            {
                key = "Profile",
                value = "car",
            };
            CallerContextProperty[] properties = { ccp1, ccp2, ccp3 };
            cxt.wrappedProperties = properties;

            // Step 2.1: Define sequencing transport depot 101
            xserver.Point point1010 = new xserver.Point();
            point1010.point = new PlainPoint { x = xy1010[0], y = xy1010[1] };
            TransportPoint transportPoint1010 = new TransportPoint
            {
                id = 1010,
                description = "Tourstop 1010",
                location = point1010,
                openingIntervalConstraint = OpeningIntervalConstraint.START_OF_SERVICE,
                allowServiceSplit = true,
                coDriverRequired = false,
                servicePeriod = 3600,
                tourSection = 0,
                useServicePeriodForRecreation = false,
            };

            Quantities quantities101 = new Quantities();
            quantities101.wrappedQuantities = new int[] { 100 };
            SequencingTransportDepot sequencingTransportDepot101 = new SequencingTransportDepot
            {
                id = 101,
                transportPoint = transportPoint1010,
                priority = 5,
                //priority = 1,
                deliveryQuantities = quantities101,
                prioritySpecified = true
            };

            // Step 2.2: Define sequencing transport depot 102
            xserver.Point point1020 = new xserver.Point();
            point1020.point = new PlainPoint { x = xy1020[0], y = xy1020[1] };
            TransportPoint transportPoint1020 = new TransportPoint
            {
                id = 1020,
                description = "Tourstop 1020",
                location = point1020,
                openingIntervalConstraint = OpeningIntervalConstraint.START_OF_SERVICE,
                allowServiceSplit = true,
                coDriverRequired = false,
                servicePeriod = 3500,
                tourSection = 0,
                useServicePeriodForRecreation = false,
            };

            Quantities quantities102 = new Quantities();
            quantities102.wrappedQuantities = new int[] { 100 };
            SequencingTransportDepot sequencingTransportDepot102 = new SequencingTransportDepot
            {
                id = 102,
                transportPoint = transportPoint1020,
                priority = 1,
                deliveryQuantities = quantities102,
                prioritySpecified = true
            };

            // Step 2.3: Define sequencing transport depot 103
            xserver.Point point1030 = new xserver.Point();
            point1030.point = new PlainPoint { x = xy1030[0], y = xy1030[1] };
            TransportPoint transportPoint1030 = new TransportPoint
            {
                id = 1030,
                description = "Tourstop 1030",
                location = point1030,
                openingIntervalConstraint = OpeningIntervalConstraint.START_OF_SERVICE,
                allowServiceSplit = true,
                coDriverRequired = false,
                servicePeriod = 3300,
                tourSection = 0,
                useServicePeriodForRecreation = false,
            };

            Quantities quantities103 = new Quantities();
            quantities103.wrappedQuantities = new int[] { 100 };
            SequencingTransportDepot sequencingTransportDepot103 = new SequencingTransportDepot
            {
                id = 103,
                transportPoint = transportPoint1030,
                priority = 1,
                deliveryQuantities = quantities103,
                prioritySpecified = true
            };

            // Step 2.4: Define sequencing transport AB 104
            xserver.Point point1041 = new xserver.Point();
            point1041.point = new PlainPoint { x = xy1041[0], y = xy1041[1] };
            TransportPoint transportPoint1041 = new TransportPoint
            {
                id = 1041,
                description = "Tourstop 1041",
                location = point1041,
                openingIntervalConstraint = OpeningIntervalConstraint.START_OF_SERVICE,
                allowServiceSplit = true,
                coDriverRequired = false,
                servicePeriod = 4000,
                tourSection = 0,
                useServicePeriodForRecreation = false,
            };

            xserver.Point point1042 = new xserver.Point();
            point1042.point = new PlainPoint { x = xy1042[0], y = xy1042[1] };
            TransportPoint transportPoint1042 = new TransportPoint
            {
                id = 1042,
                description = "Tourstop 1042",
                location = point1042,
                openingIntervalConstraint = OpeningIntervalConstraint.START_OF_SERVICE,
                allowServiceSplit = true,
                coDriverRequired = false,
                servicePeriod = 4400,
                tourSection = 0,
                useServicePeriodForRecreation = false,
            };

            Quantities quantities104 = new Quantities();
            quantities104.wrappedQuantities = new int[] { 100 };
            SequencingTransportAB sequencingTransportAB104 = new SequencingTransportAB
            {
                id = 104,
                transportPointA = transportPoint1041,
                transportPointB = transportPoint1042,
                quantities = quantities104,
                //priority = 1,
                priority = 3,
                prioritySpecified = true
            };

            // Step 2.5: Define array of transport orders
            SequencingTransportOrder[] sequencingTransportOrders = new SequencingTransportOrder[] {
				    sequencingTransportDepot101, sequencingTransportDepot102, sequencingTransportDepot103, sequencingTransportAB104 };

            // Step 3.1: Define depot 201
            xserver.Point pointDepot = new xserver.Point();
            pointDepot.point = new PlainPoint { x = xy201[0], y = xy201[1] };
            Interval openingInterval = new Interval
            {
                from = 28800,
                till = 64800,
            };
            Interval[] openingIntervals = new Interval[] { openingInterval };
            Depot depotLux = new Depot
            {
                id = 201,
                description = "Depot 201",
                location = pointDepot,
                wrappedOpeningIntervals = openingIntervals,
            };

            // Step 3.2: Define array of depots
            Depot[] depots = new Depot[] { depotLux };

            // Step 4: Define vehicle at depot location
            SequencingVehicle vehicle = new SequencingVehicle
            {
                depotIdStart = 201,
                depotIdEnd = 201,
                dimaId = 1,
                dimaIdSpecified = true
            };

            // Step 5.1: Define planning parameters
            SequencingParams planningParams = new SequencingParams();
            planningParams.coDriverTransportPointsInSeparateTours = false;

            // Step 5.2: Set properties for distance matrix.
            DistanceMatrixByRoad distanceMatrixByRoad = new DistanceMatrixByRoad
            {
                dimaId = 1,
                deleteAfterUsage = true,
                deleteBeforeUsage = true
            };
            DistanceMatrixByRoad[] dimas = new DistanceMatrixByRoad[] { distanceMatrixByRoad };
            planningParams.wrappedDistanceMatrixCalculation = dimas;

            // Step 5.3: Maximum of 6 hours is permitted for the complete tour duration.
            TourRestrictions tourRestrictions = new TourRestrictions();
            tourRestrictions.maxPeriod = 21600;
            tourRestrictions.maxPeriodSpecified = true;
            Restrictions restrictions = new Restrictions();
            restrictions.tourRestrictions = tourRestrictions;
            planningParams.restrictions = restrictions;

            // Step 6: There is no input plan used!

            try
            {
                // Step 7: Call method planSequence and add the arguments as
                // defined above.
                SequencingPlan plan = xTourClient.planSequence(sequencingTransportOrders, depots, vehicle, planningParams, null, cxt);
                return plan;
            }
            catch (Exception e)
            {
                // TODO: proper exception handling
                return null;
            }
        }
    }

    public class TourPointTableEntry
    {
        public int ID { get; set; }
        public String desc { get; set; }
        public double dist { get; set; }
        public String arrival { get; set; }
        public String driving { get; set; }
        public TourPointType type { get; set; }
    }

    public class TourEventTableEntry
    {
        public String ID { get; set; }
        public String start { get; set; }
        public String period { get; set; }
        public String type { get; set; }
    }

    public class TourOrderTableEntry
    {
        public String type { get; set; }
        public int ID { get; set; }
    }
}
